@@ -1,5 +1,5 @@
 from typing import Any
-from bergen.schema import AssignationParams
+from bergen.schema import AssignationParams, Pod, PodStatus, ProvisionParams
 from bergen.registries.arnheim import get_current_arnheim
 import asyncio
 from bergen.types.model import ArnheimModel
@@ -7,14 +7,7 @@ from bergen.extenders.base import BaseExtender
 import logging
 from aiostream import stream
 import uuid
-
-
-
-
-logger = logging.getLogger(__name__)
-
-
-
+from bergen.extenders.contexts.pod import HostedPod
 
 class NodeExtender(BaseExtender):
 
@@ -27,11 +20,22 @@ class NodeExtender(BaseExtender):
         self._postman = bergen.getPostman()
         self._loop, self._force_sync = bergen.getLoopAndContext()
 
-    async def __call_async(self, inputs: dict, params: AssignationParams, **kwargs):
+
+    def provide(self, params: ProvisionParams, **kwargs):
+        if self._loop.is_running() and not self._force_sync:
+            return self.provide_async(params, **kwargs)
+        else:
+            result = self._loop.run_until_complete(self.provide_async(params, **kwargs))
+            return result
+
+    async def provide_async(self, params: ProvisionParams, **kwargs):
+        return await self._postman.provide(self, params, **kwargs)       
+
+    async def assign_async(self, inputs: dict, params: AssignationParams, **kwargs):
         
         return await self._postman.assign(self, inputs, params, **kwargs)
 
-    async def __delay_async(self, inputs: dict, params: AssignationParams, **kwargs):
+    async def delay_async(self, inputs: dict, params: AssignationParams, **kwargs):
     
         return await self._postman.delay(self, inputs, params, **kwargs)
 
@@ -41,9 +45,9 @@ class NodeExtender(BaseExtender):
     
     def delay(self, inputs: dict, params: AssignationParams = None, **kwargs):
         if self._loop.is_running() and not self._force_sync:
-            return self.__delay_async(inputs, params, **kwargs)
+            return self.delay_async(inputs, params, **kwargs)
         else:
-            result = self._loop.run_until_complete(self.__call_async(inputs, params, **kwargs))
+            result = self._loop.run_until_complete(self.delay_async(inputs, params, **kwargs))
             return result
 
 
@@ -58,12 +62,16 @@ class NodeExtender(BaseExtender):
             outputs (dict): The ooutputs of this Node
         """
         if self._loop.is_running() and not self._force_sync:
-            return self.__call_async(inputs, params, **kwargs)
+            return self.assign_async(inputs, params, **kwargs)
         else:
-            result = self._loop.run_until_complete(self.__call_async(inputs, params, **kwargs))
+            result = self._loop.run_until_complete(self.assign_async(inputs, params, **kwargs))
             return result
 
 
+
+    def hosted(self, params: ProvisionParams = {}, enter_when = PodStatus.ACTIVE, **kwargs):
+        return HostedPod(self, params = {}, enter_when=enter_when, **kwargs)
+
     def _repr_html_(self):
         string = f"{self.name}</br>"
 
