@@ -3,4 +3,6 @@ EXCEPTION = "exception"
 ALLOWANCE = "allowance"
 
 ASSIGNATION = "assignation"
-ASSIGNATION_REQUEST = "assignation_request"
\ No newline at end of file
+PROVISION = "provision"
+ASSIGNATION_REQUEST = "assignation_request"
+PROVISION_REQUEST = "provision_request"
\ No newline at end of file
