@@ -1,9 +1,12 @@
+from bergen.extenders.pod import PodExtender
 from bergen.types.model import ArnheimModelManager
 from bergen.extenders.user import UserExtender
 from bergen.types.node.inputs import Inputs, Outputs
-from bergen.delayed import CREATE_NODE_MUTATION, NODE_FILTER_QUERY, NODE_QUERY
+from bergen.queries.delayed.node import CREATE_NODE_MUTATION, NODE_FILTER_QUERY, NODE_QUERY
+from bergen.queries.delayed.pod import POD_QUERY
 from bergen.extenders.node import NodeExtender
 from bergen.schema import Node as SchemaNode
+from bergen.schema import Pod as SchemaPod
 from bergen.schema import User as SchemaUser
 from bergen.schema import *
 try:
@@ -44,6 +47,17 @@ class Node(NodeExtender, SchemaNode):
 
 
 
+class Pod(PodExtender, SchemaPod):
+    __slots__ = ("_loop", "_force_sync", "_postman")
+
+    objects = NodeManager()
+
+    class Meta:
+        overwrite_default = True
+        identifier = "pod"
+        get = POD_QUERY
+
+
 class User(UserExtender, SchemaUser):
 
     class Meta:
