@@ -19,7 +19,17 @@ class BasePostman(ABC):
 
 
     @abstractmethod
-    async def assign(self, inputs: dict, params: dict, **kwargs):
+    async def assign(self, node, inputs: dict, params: dict, **kwargs):
+
+        return NotImplementedError("This is abstract")
+
+    @abstractmethod
+    async def provide(self, params: dict, **kwargs):
+
+        return NotImplementedError("This is abstract")
+
+    @abstractmethod
+    async def unprovide(self):
 
         return NotImplementedError("This is abstract")
 
