@@ -1,8 +1,10 @@
-
-
+from asyncio.tasks import ensure_future
+from bergen.messages.provision_request import ProvisionRequestMessage
+from bergen.utils import expandOutputs, shrinkInputs
 from bergen.messages.assignation import AssignationMessage
+from bergen.messages.provision import ProvisionMessage
 from bergen.messages.exceptions.base import ExceptionMessage
-from bergen.messages.types import ASSIGNATION, EXCEPTION
+from bergen.messages.types import ASSIGNATION, EXCEPTION, PROVISION
 from bergen.messages.base import MessageModel
 from bergen.messages.assignation_request import AssignationRequestMessage
 from bergen.postmans.base import BasePostman
@@ -10,10 +12,19 @@ from aiostream import stream
 import json
 import uuid
 import logging
+
 import asyncio
+try:
+    from asyncio import create_task
+except ImportError:
+    #python 3.6 fix
+    create_task = asyncio.ensure_future
+
+
 import websockets
 import sys
 from bergen.schema import AssignationStatus
+from bergen.models import Pod
 
 
 
@@ -21,9 +32,6 @@ logger = logging.getLogger(__name__)
 
 
 
-
-
-
 class WebsocketPostman(BasePostman):
     type = "websocket"
 
@@ -35,8 +43,16 @@ class WebsocketPostman(BasePostman):
         self.connection = None      
         self.channel = None         
         self.callback_queue = ''
+
+
         self.futures = {}
 
+
+
+
+        self.provision_futures = {}
+        self.unprovision_futures = {}
+
         self.configured = False
         self.consumer_task = None
         self.producer_task = None
@@ -48,21 +64,23 @@ class WebsocketPostman(BasePostman):
     async def configure(self):
         self.callback_queue = asyncio.Queue()
         self.progress_queue = asyncio.Queue()
-        self.startup_task = asyncio.create_task(self.startup())
+        self.startup_task = create_task(self.startup())
 
 
     async def disconnect(self):
         if self.startup_task: self.startup_task.cancel()
         if self.consumer_task: self.consumer_task.cancel()
         if self.producer_task: self.producer_task.cancel()
+        print("Bergen disconnected")
 
     async def startup(self):
 
         await self.connect_websocket()
-        self.consumer_task = asyncio.create_task(
+
+        self.consumer_task = create_task(
             self.producer()
         )
-        self.producer_task = asyncio.create_task(
+        self.producer_task = create_task(
             self.workers()
         )
 
@@ -94,16 +112,17 @@ class WebsocketPostman(BasePostman):
             message = await self.callback_queue.get()
             try:
                 parsed_message = MessageModel.from_channels(message=message)
-
+                
+                print(parsed_message)
                 # Stream Path
             
                 if parsed_message.meta.type == EXCEPTION:
                     parsed_exception = ExceptionMessage.from_channels(message=message)
-                    future = self.futures.pop(parsed_exception.data.reference)
+                    future = self.futures.pop(parsed_exception.meta.reference)
                     future.set_exception(Exception(parsed_exception.data.message))
 
 
-                if parsed_message.meta.type == ASSIGNATION:
+                elif parsed_message.meta.type == ASSIGNATION:
                     parsed_assignation = AssignationMessage.from_channels(message=message)
 
                     correlation_id = parsed_assignation.data.reference
@@ -113,7 +132,24 @@ class WebsocketPostman(BasePostman):
                     
                     if correlation_id in self.futures:
                         future = self.futures.pop(correlation_id)
-                        future.set_result(parsed_assignation.data.outputs)
+
+                        if parsed_assignation.data.status == AssignationStatus.DONE:
+                            future.set_result(parsed_assignation.data.outputs)
+                        if parsed_assignation.data.status == AssignationStatus.ERROR:
+                            future.set_exception(Exception(parsed_assignation.data.statusmessage))
+
+                elif parsed_message.meta.type == PROVISION:
+                    print(message)
+                    parsed_provision = ProvisionMessage.from_channels(message=message)
+
+                    correlation_id = parsed_provision.meta.reference
+
+                    if correlation_id in self.active_stream_queues:
+                        await self.active_stream_queues[correlation_id].put(parsed_provision)
+                    
+                    if correlation_id in self.provision_futures:
+                        future = self.provision_futures.pop(correlation_id)
+                        future.set_result(parsed_provision.data.pod)
 
                 else:
                     raise Exception("Received something weird", parsed_message )
@@ -125,10 +161,12 @@ class WebsocketPostman(BasePostman):
     async def stream(self, node, inputs, params, **extensions):
         logger.info(f"Creating a Stream of Data for {node.id}")
         correlation_id = str(uuid.uuid4())
+
+        assigned_inputs = await shrinkInputs(node=node, inputs=inputs)
         self.active_stream_queues[correlation_id] = asyncio.Queue()
         request = AssignationRequestMessage(data={
                                                 "node":node.id, 
-                                                "inputs": inputs, 
+                                                "inputs": assigned_inputs, 
                                                 "params": dict(params or {}),
                                                 "reference": correlation_id,
                                                 # We omit sending a callback
@@ -146,7 +184,7 @@ class WebsocketPostman(BasePostman):
         while True:
             messagein_stream = await self.active_stream_queues[correlation_id].get()
             if messagein_stream.data.status == AssignationStatus.YIELD:
-                yield messagein_stream.data.outputs
+                yield await expandOutputs(node, outputs=messagein_stream.data.outputs)
 
             if messagein_stream.data.status == AssignationStatus.DONE:
                 break
@@ -159,15 +197,17 @@ class WebsocketPostman(BasePostman):
             raise Exception("No longer connected. Did you use an Async context manager?")
 
 
-    async def assign(self, node, inputs, params, **extensions):
+    async def assign(self, node_or_pod, inputs, params, **extensions):
         correlation_id = str(uuid.uuid4()) # Where should we do this?
         future = self.loop.create_future()
         self.futures[correlation_id] = future
-       
 
+        assigned_inputs = await shrinkInputs(node=node_or_pod, inputs=inputs)
+       
+        #TODO: Implement assigning to pod directly
         request = AssignationRequestMessage(data={
-                                                "node":node.id, 
-                                                "inputs": inputs or {}, 
+                                                "node":node_or_pod.id, 
+                                                "inputs": assigned_inputs, 
                                                 "params": dict(params or {}),
                                                 "reference": correlation_id,
                                                 "callback": None,
@@ -184,16 +224,54 @@ class WebsocketPostman(BasePostman):
 
         await self.send_to_arnheim(request)
             
-        return await future
+        outputs = await future
+        return await expandOutputs(node_or_pod, outputs)
+
+
+    async def provide(self, node, params, **extensions):
+        correlation_id = str(uuid.uuid4()) # Where should we do this?
+        future = self.loop.create_future()
+        self.provision_futures[correlation_id] = future
+
+        request = ProvisionRequestMessage(data={
+                                                "node":node.id, 
+                                                "params": dict(params or {}),
+                                                "reference": correlation_id,
+                                            },
+                                            meta={
+                                                "reference": correlation_id,
+                                                "auth": {
+                                                    "token": self.token
+                                                },
+                                                "extensions": extensions
+                                            })
+
+
+        await self.send_to_arnheim(request)
+            
+
+        pod_id = await future
+        pod = await Pod.asyncs.get(id=pod_id)
+        return pod
+
+
+    async def unprovide(self, pod):
+        correlation_id = str(uuid.uuid4()) # Where should we do this?
+        #future = self.loop.create_future()
+        #self.unprovision_futures[correlation_id] = future
+
+        print("Unproviding not implememted yet")           
+
+        return False
 
 
     async def delay(self, node, inputs, params, **extensions):
         
         reference = str(uuid.uuid4())
-        
+        assigned_inputs = await shrinkInputs(node=node, inputs=inputs)
         request = AssignationRequestMessage(data={
                                                 "node":node.id, 
-                                                "inputs": inputs, 
+                                                "inputs": assigned_inputs, 
                                                 "params": dict(params or {}),
                                                 "reference": reference,
                                                 # We omit sending a callback
